import { Body, Controller, Get, Post, UseGuards } from '@nestjs/common';
import { CategoriesFormationsService } from '../services/categories-formations.service';
import { CreateCategoriesFormations } from '../dto/create-categoriesFormations.dto';
import { CategoriesFormations } from '../schemas/categoriesFormations.schema';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';
import { Roles } from '../../users/decorators/roles.decorator';
import { RolesGuard } from '../../users/guards/roles.guard';

@Controller('categories/categoriesFormations')
export class CategoriesFormationsController {
  constructor(private readonly categoriesFormationsService: CategoriesFormationsService) {}

  @Post()
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Roles('admin')
  public async create(@Body() createStructureDto: CreateCategoriesFormations): Promise<CategoriesFormations> {
    return this.categoriesFormationsService.create(createStructureDto);
  }

  @Get()
  public async findAll(): Promise<CategoriesFormations[]> {
    return this.categoriesFormationsService.findAll();
  }
}
