import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { CreateCategoriesFormations } from '../dto/create-categoriesFormations.dto';
import { CategoriesFormationsService } from './categories-formations.service';

describe('CategoriesFormationsService', () => {
  let service: CategoriesFormationsService;

  const mockCategoriesFormationModel = {
    create: jest.fn(),
    find: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        CategoriesFormationsService,
        {
          provide: getModelToken('CategoriesFormations'),
          useValue: mockCategoriesFormationModel,
        },
      ],
    }).compile();

    service = module.get<CategoriesFormationsService>(CategoriesFormationsService);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  it('should create categorie formation', async () => {
    const data: CreateCategoriesFormations = {
      name: 'test',
      modules: [
        {
          id: '2',
          display_id: '2',
          display_name: 'Modules APTIC - n°2',
          text: 'Les conduites à risques et les bons usages du numérique',
          digest: 'Les conduites à risques et les bons usages du numérique - 02',
        },
      ],
    };
    const _doc = { _id: '5fbb92ef80a5c257dc0161f2', ...data };
    mockCategoriesFormationModel.create.mockResolvedValueOnce(_doc);
    expect(await service.create(data)).toEqual(_doc);
  });

  it('should findAll structures', async () => {
    const data: CreateCategoriesFormations[] = [
      {
        name: 'test',
        modules: [
          {
            id: '2',
            display_id: '2',
            display_name: 'Modules APTIC - n°2',
            text: 'Les conduites à risques et les bons usages du numérique',
            digest: 'Les conduites à risques et les bons usages du numérique - 02',
          },
        ],
      },
      {
        name: 'test',
        modules: [
          {
            id: '1',
            display_id: '1',
            display_name: 'Modules APTIC - n°1',
            text: 'Les conduites à risques et les bons usages',
            digest: 'Les conduites à risques et les bons usages - 01',
          },
        ],
      },
    ];
    mockCategoriesFormationModel.find.mockResolvedValueOnce(data);
    expect(await service.findAll()).toEqual(data);
  });
});
