import { Logger } from '@nestjs/common';
import * as dotenv from 'dotenv';
import { config } from './config';
import { configProd } from './config.prod';
import { configDev } from './config.dev';
import { configRec } from './config.rec';
export class ConfigurationService {
  private readonly _config;

  constructor() {
    // Initializing conf with values from var env
    switch (process.env.NODE_ENV) {
      case 'production':
        this._config = configProd;
        Logger.log('App started with production conf', 'ConfigurationService');
        break;
      case 'dev':
        this._config = configDev;
        Logger.log('App started with dev conf', 'ConfigurationService');
        break;
      case 'rec':
        this._config = configRec;
        Logger.log('App started with rec conf', 'ConfigurationService');
        break;
      default:
        this._config = config;
        Logger.warn('App started with default conf', 'ConfigurationService');
        break;
    }
    this._config.templates = config.templates; // Add mail templates
    dotenv.config();
  }

  public isLocalConf(): boolean {
    return process.env.NODE_ENV === 'local';
  }

  get config() {
    return this._config;
  }
}
