import { Db } from 'mongodb';
import { getDb } from '../migrations-utils/db';

export const up = async () => {
  const db: Db = await getDb();

  const cursor = db.collection('structures').find({});
  let document;
  while ((document = await cursor.next())) {
    const newDoc = updateStructure(document);
    await db
      .collection('structures')
      .updateOne({ _id: document._id }, [
        { $set: newDoc },
        { $unset: ['equipmentsDetails', 'nomDeLusager', 'statutJuridique', 'documentsMeeting'] },
      ]);
  }
  console.log(`Update done`);
};

export const down = async () => {
  const db: Db = await getDb();

  const cursor = db.collection('structures').find({});
  let document;
  while ((document = await cursor.next())) {
    const newDoc = downgradeStructure(document);
    await db.collection('structures').updateOne({ _id: document._id }, [{ $set: newDoc }]);
  }
  console.log(`Update done`);
};

function updateStructure(doc) {
  return updateHours(doc);
}

function downgradeStructure(doc) {
  doc = restoreHours(doc);
  return doc;
}

function updateHours(doc) {
  if (doc.hours) {
    Object.keys(doc.hours).forEach((key) => {
      if (doc.hours[key].time.length > 0) {
        doc.hours[key].time.forEach((timeRange) => {
          timeRange.openning = formatHours(timeRange.openning);
          timeRange.closing = formatHours(timeRange.closing);
        });
      }
    });
    return doc;
  } else {
    console.warn(`No hours on doc ${doc._id}`);
    return doc;
  }
}

function restoreHours(doc) {
  if (doc.hours) {
    Object.keys(doc.hours).forEach((key) => {
      if (doc.hours[key].time.length > 0) {
        doc.hours[key].time.forEach((timeRange) => {
          timeRange.openning = formatBackHours(timeRange.openning);
          timeRange.closing = formatBackHours(timeRange.closing);
        });
      }
    });
    return doc;
  } else {
    console.warn(`No hours on doc ${doc._id}`);
    return doc;
  }
}

function formatHours(hour): string {
  const stringifiedHour = hour.toString();
  if (stringifiedHour.length === 3) {
    // 930
    return stringifiedHour.slice(0, 1) + ':' + stringifiedHour.slice(1, 3);
  } else if (stringifiedHour.length === 4) {
    // 1200
    return stringifiedHour.slice(0, 2) + ':' + stringifiedHour.slice(2, 4);
  }
}

function formatBackHours(hour): number {
  const splitedHour = hour.split(':');
  return parseInt(''.concat(...splitedHour), 10);
}
