import { Db } from 'mongodb';
import { getDb } from '../migrations-utils/db';

export const up = async () => {
  const db: Db = await getDb();

  const cursor = db.collection('structures').find({});
  let document;
  while ((document = await cursor.next())) {
    const newDoc = updateStructure(document);
    await db.collection('structures').updateOne({ _id: document._id }, [{ $set: newDoc }]);
  }
  console.log(`Update done`);
};

export const down = async () => {
  const db: Db = await getDb();

  const cursor = db.collection('structures').find({});
  let document;
  while ((document = await cursor.next())) {
    const newDoc = downgradeStructure(document);
    await db.collection('structures').updateOne({ _id: document._id }, [{ $set: newDoc }]);
  }
  console.log(`Update done`);
};

function updateStructure(doc) {
  return updateHours(doc);
}

function downgradeStructure(doc) {
  return restoreHours(doc);
}

function updateHours(doc) {
  if (doc.hours) {
    Object.keys(doc.hours).forEach((key) => {
      doc.hours[key].time.forEach((timeRange) => {
        timeRange.opening = timeRange.openning;
        delete timeRange.openning;
      });
    });
    return doc;
  } else {
    console.warn(`No hours on doc ${doc._id}`);
    return doc;
  }
}

function restoreHours(doc) {
  if (doc.hours) {
    Object.keys(doc.hours).forEach((key) => {
      if (doc.hours[key].time.length > 0) {
        doc.hours[key].time.forEach((timeRange) => {
          timeRange.openning = timeRange.opening;
          delete timeRange.opening;
        });
      }
    });
    return doc;
  } else {
    console.warn(`No hours on doc ${doc._id}`);
    return doc;
  }
}
