import { Db } from 'mongodb';
import { getDb } from '../migrations-utils/db';

export const up = async () => {
  const db: Db = await getDb();

  const cursor = db.collection('structures').find({});
  let document;
  while ((document = await cursor.next())) {
    const newDoc = updateStructure(document);
    await db.collection('structures').updateOne({ _id: document._id }, [{ $set: newDoc }]);
  }
  console.log(`Update done`);
};

export const down = async () => {
  const db: Db = await getDb();

  const cursor = db.collection('structures').find({});
  let document;
  while ((document = await cursor.next())) {
    const newDoc = downgradeStructure(document);
    await db.collection('structures').updateOne({ _id: document._id }, [{ $set: newDoc }]);
  }
  console.log(`Update done`);
};

function updateStructure(doc) {
  return updateTimestamp(doc);
}

function downgradeStructure(doc) {
  return restoreTimestamp(doc);
}

function updateTimestamp(doc) {
  doc.createdAt = new Date(doc.createdAt).toISOString();
  doc.updatedAt = new Date(doc.updatedAt).toISOString();
  return doc;
}

function restoreTimestamp(doc) {
  doc.createdAt = new Date(doc.createdAt).toString();
  doc.updatedAt = new Date(doc.updatedAt).toISOString();
  return doc;
}
