import { Prop, Schema, SchemaFactory } from '@nestjs/mongoose';
import { Type } from 'class-transformer';
import { ArrayNotEmpty, IsNotEmpty, ValidateNested } from 'class-validator';
import { Document } from 'mongoose';
import { Address } from './address.schema';
import { Week } from './week.schema';

export type StructureDocument = Structure & Document;

@Schema({ timestamps: true })
export class Structure {
  constructor(data: Partial<Structure> = {}) {
    this.structureName = data.structureName;
    this.createdAt = data.createdAt;
    this.updatedAt = data.updatedAt;
    this.numero = data.numero;
    this.structureType = data.structureType;
    this.description = data.description;
    this.lockdownActivity = data.lockdownActivity;
    this.address = data.address;
    this.contactPhone = data.contactPhone;
    this.contactMail = data.contactMail;
    this.website = data.website;
    this.facebook = data.facebook;
    this.twitter = data.twitter;
    this.instagram = data.instagram;
    this.linkedin = data.linkedin;
    this.pmrAccess = data.pmrAccess;
    this.accessModality = data.accessModality;
    this.otherDescription = data.otherDescription;
    this.labelsQualifications = data.labelsQualifications;
    this.publics = data.publics;
    this.exceptionalClosures = data.exceptionalClosures;
    this.publicsAccompaniment = data.publicsAccompaniment;
    this.proceduresAccompaniment = data.proceduresAccompaniment;
    this.remoteAccompaniment = data.remoteAccompaniment;
    this.baseSkills = data.baseSkills;
    this.accessRight = data.accessRight;
    this.socialAndProfessional = data.socialAndProfessional;
    this.parentingHelp = data.parentingHelp;
    this.exceptionalClosures = data.exceptionalClosures;
    this.digitalCultureSecurity = data.digitalCultureSecurity;
    this.equipmentsAndServices = data.equipmentsAndServices;
    this.freeWorkShop = data.freeWorkShop;
    this.nbTablets = data.nbTablets;
    this.nbPrinters = data.nbPrinters;
    this.nbComputers = data.nbComputers;
    this.nbNumericTerminal = data.nbNumericTerminal;
    this.nbScanners = data.nbScanners;
    this.hours = data.hours;
    this.coord = data.coord;
    this.deletedAt = data.deletedAt;
    this.accountVerified = data.accountVerified;
  }

  @Prop()
  numero: string;

  @Prop()
  createdAt: string;

  @Prop()
  updatedAt: string;

  @Prop()
  @IsNotEmpty()
  structureName: string;

  @Prop()
  @IsNotEmpty()
  structureType: string;

  @Prop()
  description: string;

  @Prop()
  lockdownActivity: string;

  @Prop()
  @ValidateNested({ each: true })
  @Type(() => Address)
  address: Address;

  @Prop()
  @IsNotEmpty()
  contactPhone: string;

  @Prop()
  @IsNotEmpty()
  contactMail: string;

  @Prop()
  website: string;

  @Prop()
  facebook: string;

  @Prop()
  twitter: string;

  @Prop()
  instagram: string;

  @Prop()
  linkedin: string;

  @Prop()
  @IsNotEmpty()
  pmrAccess: boolean;

  @Prop()
  @ArrayNotEmpty()
  accessModality: string[];

  @Prop()
  otherDescription: string;

  @Prop()
  labelsQualifications: string[];

  @Prop()
  @ArrayNotEmpty()
  publics: string[];

  @Prop()
  exceptionalClosures: string;

  @Prop()
  publicsAccompaniment: string[];

  @Prop()
  proceduresAccompaniment: string[];

  @Prop()
  @IsNotEmpty()
  remoteAccompaniment: boolean;

  @Prop()
  baseSkills: string[];

  @Prop()
  accessRight: string[];

  @Prop()
  socialAndProfessional: string[];

  @Prop()
  parentingHelp: string[];

  @Prop()
  digitalCultureSecurity: string[];

  @Prop()
  equipmentsAndServices: string[];

  @Prop()
  @IsNotEmpty()
  freeWorkShop: boolean;

  @Prop()
  nbComputers: number;

  @Prop()
  @IsNotEmpty()
  nbPrinters: number;

  @Prop()
  @IsNotEmpty()
  nbTablets: number;

  @Prop()
  @IsNotEmpty()
  nbNumericTerminal: number;

  @Prop()
  @IsNotEmpty()
  nbScanners: number;

  @Prop()
  hours: Week;

  @Prop()
  coord: number[];

  @Prop()
  deletedAt: Date;

  @Prop()
  accountVerified: boolean;
}

export const StructureSchema = SchemaFactory.createForClass(Structure);
StructureSchema.index({ '$**': 'text' });
