import { HttpModule, HttpStatus } from '@nestjs/common';
import { ConfigModule } from '@nestjs/config';
import { ElasticsearchService } from '@nestjs/elasticsearch';
import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { UsersServiceMock } from '../../../test/mock/services/user.mock.service';
import { ConfigurationService } from '../../configuration/configuration.service';
import { MailerModule } from '../../mailer/mailer.module';
import { MailerService } from '../../mailer/mailer.service';
import { SearchModule } from '../../search/search.module';
import { UsersService } from '../../users/users.service';
import { structureDto } from '../dto/structure.dto';
import { Structure } from '../schemas/structure.schema';
import { StructuresSearchService } from './structures-search.service';
import { StructuresService } from './structures.service';
describe('StructuresService', () => {
  let service: StructuresService;

  const mockStructureModel = {
    create: jest.fn(),
    deleteOne: jest.fn(),
    countDocuments: jest.fn(),
    findOne: jest.fn(),
    exec: jest.fn(),
    find: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [HttpModule, MailerModule, SearchModule, ConfigModule],
      providers: [
        StructuresService,
        ConfigurationService,
        StructuresSearchService,
        {
          provide: getModelToken(Structure.name),
          useValue: mockStructureModel,
        },
        {
          provide: UsersService,
          useClass: UsersServiceMock,
        },
      ],
    }).compile();

    service = module.get<StructuresService>(StructuresService);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  it('should Initiate structure', () => {
    const res = service.initiateStructureIndex();
    expect(res).toBeTruthy();
  });

  it('should searchForStructures', () => {
    let res = service.searchForStructures('a', [{ nbPrinters: '1' }]);
    expect(res).toBeTruthy();
    res = service.searchForStructures('a');
    expect(res).toBeTruthy();
  });

  it('should create structure', () => {
    const structure = new structureDto();
    let res = service.create(null, structure);
    expect(res).toBeTruthy();
    res = service.create('tsfsf6296', structure);
    expect(res).toBeTruthy();
  });

  it('should search structure', () => {
    const filters = [{ nbPrinters: '1' }];
    let res = service.search('', filters);
    expect(res).toBeTruthy();
    res = service.search(null, filters);
    expect(res).toBeTruthy();
    res = service.search(null);
    expect(res).toBeTruthy();
  });

  it('should find all structures', () => {
    const res = service.findAll();
    expect(res).toBeTruthy();
  });

  it('should find all unclaimed structures', () => {
    const res = service.findAllUnclaimed();
    expect(res).toBeTruthy();
  });

  it('should find all formated structures', () => {
    const res = service.findAllFormated(null, null, null);
    expect(res).toBeTruthy();
  });

  it('should populate ES', () => {
    const res = service.populateES();
    expect(res).toBeTruthy();
  });

  it('should report structure Error', () => {
    let res = service.reportStructureError('6093ba0e2ab5775cfc01ed3e', '');
    expect(res).toBeTruthy();
    res = service.reportStructureError(null, '');
    expect(res).toBeTruthy();
  });
});
