import { Reflector } from '@nestjs/core';
import { createMock } from '@golevelup/ts-jest';
import { Test, TestingModule } from '@nestjs/testing';
import { ExecutionContext } from '@nestjs/common';
import { UserRole } from '../enum/user-role.enum';
import { IsStructureOwnerGuard } from './isStructureOwner.guard';
import { Types } from 'mongoose';

describe('isStrructureOwner', () => {
  let guard: IsStructureOwnerGuard;
  let reflector: Reflector;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        IsStructureOwnerGuard,
        {
          provide: Reflector,
          useValue: {
            constructor: jest.fn(),
            get: jest.fn(),
          },
        },
      ],
    }).compile();

    guard = module.get<IsStructureOwnerGuard>(IsStructureOwnerGuard);
    reflector = module.get<Reflector>(Reflector);
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(guard).toBeDefined();
  });

  it('should return true if structure is in user linked structures', async () => {
    const context = createMock<ExecutionContext>({
      getHandler: jest.fn(),
      switchToHttp: jest.fn().mockReturnValueOnce({
        getRequest: jest.fn().mockReturnValueOnce({
          user: {
            structuresLink: ['6001a38516b08100062e4161'],
            structureOutdatedMailSent: [],
            pendingStructuresLink: [],
            newEmail: null,
            changeEmailToken: null,
            role: 0,
            resetPasswordToken: null,
            validationToken: null,
            emailVerified: true,
            email: 'jp@test.com',
            name: 'Jean-Paul',
            surname: 'DESCHAMPS',
            phone: '06 11 11 11 11',
          },
          params: {
            id: '6001a38516b08100062e4161',
          },
        }),
      }),
    });
    const result = await guard.canActivate(context);

    expect(result).toBeTruthy();
  });

  it('should return false if structure is not user linked structures', async () => {
    jest.spyOn(reflector, 'get').mockImplementation((a: any, b: any) => []);
    const context = createMock<ExecutionContext>({
      getHandler: jest.fn(),
      switchToHttp: jest.fn().mockReturnValue({
        getRequest: jest.fn().mockReturnValue({
          user: {
            structuresLink: ['unIdQuiExistePasTropTrop'],
            structureOutdatedMailSent: [],
            pendingStructuresLink: [],
            newEmail: null,
            changeEmailToken: null,
            role: UserRole.user,
            resetPasswordToken: null,
            validationToken: null,
            emailVerified: true,
            email: 'jp@test.com',
            name: 'Jean-Paul',
            surname: 'DESCHAMPS',
            phone: '06 11 11 11 11',
          },
          params: {
            id: '6001a38516b08100062e4161',
          },
        }),
      }),
    });
    const result = await guard.canActivate(context);

    expect(result).toBeFalsy();
    expect(reflector.get).toBeCalled();
  });
});
